require 'spec_helper'
require 'lib/common'
require 'version-one/query'

describe VersionOne::Asset do
  include_context 'configured'

  let(:meta) { VersionOne::Meta.new(load_xml('story_meta')) }

  before :each do
    allow(VersionOne::Meta).to receive(:get).and_return(meta)
  end

  shared_examples 'new' do

    it 'should succeed' do
      expect(subject).not_to be_nil
    end

    it 'should provide attribute readers from meta' do
      expect(subject).to respond_to :estimate, :number, :order
    end

    it 'should provide attribute writers from meta' do
      expect(subject).to respond_to :estimate=
    end

    it 'should reject using attribute writers from meta for read-only after init' do
      expect(subject).to respond_to :number=
      expect(lambda{ subject.number = '12345' }).to raise_error(/Can't set read-only/)
    end

    it 'should provide relation readers from meta' do
      expect(subject).to respond_to :security_scope, :status
    end

    it 'should provide relation writers from meta' do
      expect(subject).to respond_to :security_scope=, :status=
    end

  end

  describe 'new from xml' do
    context 'with a single result' do
      subject { described_class.from_xml(load_xml('story')) }

      include_examples 'new'

      it 'should set attribute values from xml' do
        expect(subject.number).to eq 'B-01027'
        expect(subject.order).to eq -1071957351
      end

      it 'should provide attribute readers for queried attributes' do
        expect(subject).to respond_to :security_scope_name
      end

      it 'should set empty single relations to nil' do
        expect(subject.team).to be_nil
      end

      it 'should set empty multiple relations to an empty RelationMultiValue' do
        expect(subject.owners).to be_a VersionOne::RelationMultiValue
        expect(subject.owners).to be_empty
      end

      it 'should set empty simple values to nil' do
        expect(subject.original_estimate).to be_nil
      end
    end

    context 'with multiple results' do
      subject { described_class.from_xml(load_xml('expression_results')) }

      it 'should return an array of assets' do
        expect(subject).to be_a Array
        expect(subject.size).to eq 2
      end

      describe 'should populate indirectly' do

        it 'single values' do
          expect(subject[0].some_thing_single_value).to eq 'Expected Value'
        end

        it 'multiple values' do
          expect(subject[0].in_reply_to_mentions_name).to eq ['Bob One', 'Bob Two', 'Bob Three', 'Bob Four', 'Some bug in the product']
        end

        it 'single relations' do
          expect(subject[0].some_thing_single_relation.size).to be > 0
          expect(subject[0].some_thing_single_relation.map(&:id)).to eq ['Member:14494']
        end

        it 'multiple relations' do
          expect(subject[0].in_reply_to_mentions.size).to be > 0
          expect(subject[0].in_reply_to_mentions.map(&:id)).to eq %w{Member:1008 Member:1120 Member:8050 Member:14494 Defect:72081}
        end

      end

    end
  end



  describe 'new' do
    subject { described_class.new(type: 'Story') }

    include_examples 'new'
  end

  describe 'save' do
    subject { described_class.new(type: 'Story') }
    let(:project) { VersionOne::Query.projects.find(config['test']['scope']) }

    include_examples 'new'

    it 'should save and update a new item' do
      subject.name = 'IGNORE. This is a V1 automation unit test.'
      subject.description = "<p><b>IGNORE.</b> This is a unit test.</p>"
      subject.scope = project
      expect(subject.save).to be_truthy
      expect(subject.id).to match /^Story(:\d{1,6})+$/
      expect(subject.href).to be_a String

      new_item = VersionOne::Query.primary_work_items.find(subject.id)
      expect(new_item).to_not be_nil
      expect(new_item.number).to match /^B-\d{5,6}$/
      expect(new_item.description).to eq subject.description
      new_item.description = "<p>Description update</p>"
      expect(new_item.save).to be_truthy

      final_item = VersionOne::Query.primary_work_items.find(subject.id)
      expect(final_item.description).to eq new_item.description
    end

  end

end
